import os
from json import loads, dumps

try:
    STATE_DIR = os.environ['SENSATE_STATE_DIR']
except:
    STATE_DIR = os.path.join(os.path.expanduser('~'), '.sensate')

## WARNING: NON-TESTED CODE PORTED FROM ANOTHER THING ##

def _require_dir(loc):
    '''Make sure that loc is a directory. If it does not exist, create it.'''
    if os.path.exists(loc):
        if not os.path.isdir(loc):
            raise ValueError('%s must be a directory' % loc)
    else:
        os.makedirs(loc, 0755)

def save_state(state_dict, filename=None, state_path=STATE_DIR):
    _require_dir(state_path)
    
    # an idea for another day.
    #state_dict = experiment.to_dict()
    
    if filename==None:
        filename = experiment.name
    
    buf = dumps(state_dict, indent=2, separators=(',', ': '))
    statefile = os.path.join(state_path, filename)
    open(statefile, 'w').write(buf)
    return True

def restore_state(subnet, state_path='/tmp/labsensors'):
    _require_dir(state_path)
    statefile = statefile_path(subnet, state_path)
    buf=open(statefile).read()
    return loads(buf)



from datetime import datetime
import json

from sensorcheck import SensorCheck
from utils import CustomJsonEncoder, CustomJsonDecoder

# TODO: make equipment swappable (when more equipment presents itself)
from equipment.wsg30 import get_sensor_oid
from equipment.wsg30 import STATUS_TROUBLE, ALARM_HIGH, ALARM_LOW, STATUS_OK, BATTERY_LOW
from transports.snmp import SNMPobject

# MAX_SENSORS_PER_HUB restricts SNMP guesswork to a reasonable level.
# (theoretical max # of sensors per WSG30 hub = 30 wireless + 1 Power + 1 Battery.)
MAX_SENSORS_PER_HUB = 32 

class SensorHub(object):
    '''The central communications hub that sensors report back to. 

    Uses SNMP to poll the hub directly over the network. 
    
    Required argument:  
    
    hostname                -- (string) fqdn or LAN-routable hostname (no protocol prefix)
    
    Optional keyword arguments:
    
    match           -- (string) restrict results to only those sensors with `match` in their name
    auto            -- (bool) whether to perform SNMP connections immediately (default: True)
    port            -- (int) port used to connect via snmp (default: 161)
    timeout         -- (int or float) time in seconds to wait for SNMP response.
    retries			-- (int) number of times to retry connection (default: 0)
    snmp_method     -- (string) if pysnmp isn't working for you, try 'cli' (default: 'pysnmp')
    
    Currently decoration-only attributes that can be set by keyword arguments:
    
    model           -- (string) defines model of Sensaphone equipment (default: WSG30)
    ''' 
    
    def __init__(self, hostname, **kwargs):
        '''
        Constructor for SensorHub.

        Uses SNMP to poll the hub directly over the network.
           
        Supply "match" to restrict results to only those sensors
        with "match" in their name.

        :param hostname: (required) fqdn or reachable local hostname.
        :param match: (optional) only load sensors where match (string) can be found in name.
        '''
    
        self.hostname = hostname        
        self.match = kwargs.get('match', '')
        self.model = str(kwargs.get('model', 'WSG30')).upper()
        
        # whether to complete SNMP checks immediately
        auto = kwargs.get('auto', True)
        
        ## SNMP ##
        self.snmp_method = kwargs.get('snmp_method', 'pysnmp')
        self.retries = kwargs.get('retries', 0)
        # generic timeout length in seconds for any type of connection.
        self.timeout = kwargs.get('timeout', 10)  
        
        # Updated every time snmp_check_sensors used.
        self.lastchecktime = None

        # Assign self.snmp_get to a generated snmp get function.  
        self.snmp_get = SNMPobject(hostname=self.hostname, timeout=self.timeout,
            			retries=self.retries, method=self.snmp_method).snmp_getter()

        self.sensors_by_idx = {}   # idx: name mapping. Filled during discover()
        self.checks_by_idx = {}    # idx: SensorCheck mapping. Filled during check()
        self.checks_by_serial = {} # serial: SensorCheck mapping. Filled during check()
        self.checks = []
        self.alarms = { 'high': [], 'low': [], 'low_battery': [], 'trouble': [] }

        if auto:
            self.reload()

    def _process_alarms(self, check):
        if check.alarm_status == BATTERY_LOW:
            self.alarms['low_battery'].append(check)
        elif check.alarm_status == STATUS_TROUBLE:
            self.alarms['trouble'].append(check)
        elif check.alarm_status == ALARM_HIGH:
            self.alarms['high'].append(check)
        elif check.alarm_status == ALARM_LOW:
            self.alarms['low'].append(check)

    def check(self):
        '''Populates and returns self.checks by retrieving SensorCheck objects via SNMP (destructive update)
        
        Also sets up self.checks_by_idx (idx to SensorCheck mapping) and self.checks_by_serial
        (they have both been useful in different contexts, so let's have both.)
        ''' 
        self.checks = []
        self.checks_by_idx = {}
        self.checks_by_serial = {}
        
        for idx in self.sensors_by_idx.keys():
            checktime = datetime.utcnow()
            check = SensorCheck.from_snmp(self.hostname, idx, name=self.sensors_by_idx[idx], checktime=checktime)
            self.checks.append(check)
            self.checks_by_idx[idx] = check
            # ignore Power and Battery for checks_by_serial mapping.
            if check.serial != '0':
                self.checks_by_serial[check.serial] = check
            self._process_alarms(check)

    def discover(self):
        '''Populates and returns self.sensors_by_idx (destructive update)
        
        :rtype: dictionary mapping of sensor index (int) to name (str) 
        '''
        self.sensors_by_idx = {} 
        self.lastchecktime = datetime.utcnow()
        
        for idx in range(MAX_SENSORS_PER_HUB+1):
            name = self.snmp_get(get_sensor_oid(idx, 'name'))
            # we sometimes get garbage characters in the sensor name :(
            try:
                name.decode()
            except UnicodeDecodeError, e:
                #TODO: CONVERT name (don't just give up on it)
                continue
            except AttributeError, e:
                # name was returned as None
                continue

            if self.match:
                if self.match in name:
                    self.sensors_by_idx[idx] = name
            else:
                if name and name != 'Unconfigured':
                    self.sensors_by_idx[idx] = name
        return self.sensors_by_idx

    def reload(self):
        '''Convenience interface to self.discover() and self.check()'''
        self.discover()
        self.check()
    
    def __str__(self):
        out = 'Sensaphone Hub at %s\n\n' % self.hostname
        for idx in sorted(self.sensors_by_idx.keys()):
            out += '%d: %s\n' % (idx, self.sensors_by_idx[idx])
        return out
    
    def to_dict(self, by_idx=False):
        '''returns dictionary representing hub and all checked sensors.
        
        if self.check() has successfully run, the 'sensors' key will have 
        self.checks_by_serial as its value, or self.checks_by_idx if by_idx=True.
        
        if only self.discover() has run, the 'sensors' key = self.sensors_by_idx
        '''
        
        if self.checks_by_idx:
            if by_idx:
                sensord = dict([(k,v.to_dict()) for k,v in self.checks_by_idx.items()])
            else:
                sensord = dict([(k,v.to_dict()) for k,v in self.checks_by_serial.items()])
        else:
            sensord = self.sensors_by_idx

        return { 'hostname': self.hostname,
                 'lastchecktime': self.lastchecktime,
                 'match': self.match,
                 'model': self.model,
                 'sensors': sensord,
                 'alarms': self.alarms
               }

    def to_json(self):
        return json.dumps(self.to_dict(), cls=CustomJsonEncoder)



import json
import datetime

import pytz
from pyrfc3339 import generate, parse

DATE_FORMAT = '%Y-%m-%dT%H:%M:%SZ'

# The following helper classes assist in datetime conversion between Python and JSON. 
class CustomJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'isoformat'):
            return generate(obj.replace(tzinfo=pytz.utc))
        else:
            return str(obj)
        return json.JSONEncoder.default(self, obj)

class CustomJsonDecoder(json.JSONDecoder):        
    def decode(self, json_string):
        obj = super(CustomJsonDecoder,self).decode(json_string)
        for k, v in obj.items():
            try:
                obj[k] = parse(v)
            except ValueError:
                pass
        return obj

if __name__=='__main__':    
    now = datetime.datetime.utcnow()
    expected_string = now.strftime(DATE_FORMAT)   
    sample = {'now': now}
    result_string = json.dumps(sample, cls=CustomJsonEncoder)
    
    assert result_string.find(expected_string) != -1

    result_dict = json.loads(result_string, cls=CustomJsonDecoder)

    print now
    print result_dict['now']

    
    


__doc__='''The SensaphoneRecord object encapsulates the historical records 
currently stored on the Sensaphone device.

This object assists in populating historical records of sensor values.

The input data structure is a comma-separated CSV containing records for either a single 
selected sensor or for all sensors.  (see below copied from the Sensaphone WSG30 manual)

From there, we construct SensorCheck objects which can be converted to 

    # dict of SensorCheck objects (similar to SensorHub.sensor_check attribute)

The challenge here is that the sensaphone stores data based on the POSITION ("idx")
of the sensors on the device in its internal reckoning. Therefore the records don't 
provide a contiguous record of monitoring in real-world terms -- we have to piece
that back together ourselves.
'''

### SNMP helpers used by hub.py and sensor.py

# Magic SNMP number for Sensaphone WSG30, used in constructing Object IDs for sensor value lookups.
SENSAPHONE_OIDBASE = '.1.3.6.1.4.1.8338.1.1.4.1.1'

SENSOR_DATA_MAP = { 'name': 15, 
                    'units': 19,
                    'alarmlow': 11,
                    'alarmhigh': 12,
                    'measurement': 48,
                    'warnlow': 74,              # STRING: "3.7C"   ?? warn at low value ?? 
                    'warnhigh': 75,             # STRING: "7.6C"   ?? warn at high value ?? 
                    'alarm_status': 91,         # STRING: "OK", "Low", "High", "Low Battery" or "Trouble"
                    'measurement_type': 92,     # STRING: 'Temp 2.8k C' or other type.  
                    'battery_level': 93,        # STRING: "4.5"     ?? battery level ??
                    'lastalarm': 98,            # STRING: "3/3/2014 1:31:14am"
                    'serial': 35,               # STRING: "13500" 
                  }

STATUS_OK = 'OK'
STATUS_TROUBLE = 'Trouble'
BATTERY_LOW = 'Low Battery'
ALARM_HIGH = 'High'
ALARM_LOW = 'Low'

def get_sensor_oid(idx, datum=''):
    '''Returns OID for individual sensor (or, if `datum` supplied, the named data point for 
    sensor) at index `idx`.  Uses global SENSAPHONE_OIDBASE to construct and return full OID.
    
    Named data points:
    
    %r

    :param idx: (required) the "index" (1-32) of the sensor on the hub.
    :param datum: (optional) 
    :return oid (string) allowing retrieval of named data point.
    ''' % SENSOR_DATA_MAP

    if datum:
        return '%s.1.%i.%i' % (SENSAPHONE_OIDBASE, idx, SENSOR_DATA_MAP[datum])
    else:
        return '%s.1.%i' % (SENSAPHONE_OIDBASE, idx)



from setuptools import setup, find_packages

setup(
    name = "sensate",
    version = "0.1.2.1",
    description = "Sensaphone hub and sensor abstraction library",
    url="https://bitbucket.org/invitae/sensate",
    author = "InVitae Inc.",
    author_email = "developers@invitae.com",
    maintainer = "Naomi Most",
    maintainer_email = "naomi.most@invitae.com",
    license = "MIT",
    zip_safe = False,
    packages = find_packages(),
    install_requires = [ 
        'setuptools',
        'pysnmp',
        'pyRFC3339'
    ],
)



__doc__='''SNMPobject class and helper functions.
 
Relies either on pysnmp (default) or snmpget on the command line (see snmp_get method).

pysnmp is used to avoid having to install snmp command line tools (also because pysnmp is
a pure-python implementation of the SNMP protocol).  The pysnmp commands are modeled on the
following magic SNMP incantations:

# Retrieve a single value:
#
# snmpget -v 2c -c public -OvQ sensaphone1.458.locusdev.net:161 .1.3.6.1.4.1.8338.1.1.4.1.1.2.15
# or
# snmpget -v2c -c public -ObentU sensaphone1.458.locusdev.net 1.3.6.1.4.1.8338.1.1.4.1.1.1.2.15
#
# Walk the whole sensaphone:
#
# snmpwalk -v 2c -m ALL -Os -c public sensaphone1.458.locusdev.net:161 1.3.6.1.4.1.8338.1.1.4
'''
__author__='nthmost'

from subprocess import Popen, PIPE
from socket import gethostbyname, gaierror
from pysnmp.entity.rfc3413.oneliner import cmdgen

SNMP_DEFAULT_PORT = 161
SNMP_DEFAULT_PROTOCOL = '2c'
SNMP_DEFAULT_COMMUNITY = 'public'
SNMP_DEFAULT_TIMEOUT = 3
SNMP_DEFAULT_RETRIES = 0
SNMP_DEFAULT_HOSTNAME = None
SNMP_DEFAULT_METHOD = 'pysnmp'       # 'pysnmp' or 'cli'

class SNMPobject(object):
    def __init__(self, *args, **kwargs):
        self.hostname = kwargs.get('hostname', SNMP_DEFAULT_HOSTNAME) 
        self.port = kwargs.get('port', SNMP_DEFAULT_PORT)
        self.protocol = str(kwargs.get('protocol', SNMP_DEFAULT_PROTOCOL))
        self.community = kwargs.get('community', SNMP_DEFAULT_COMMUNITY)
        self.timeout = kwargs.get('timeout', SNMP_DEFAULT_TIMEOUT)
        self.retries = kwargs.get('retries', SNMP_DEFAULT_RETRIES)
        self.method = kwargs.get('method', SNMP_DEFAULT_METHOD)
        
        self._results = {}      # keep a store of previously acquired results
        
        if self.protocol not in ['1', '2c', '3']:
            raise Exception('Invalid protocol "%s" supplied to SNMPobject' % self.protocol)

        # set up a pysnmp CommandGenerator to use repeatedly.
        if self.method=='pysnmp':
            self._cmdGen = cmdgen.CommandGenerator()
        else:
            self._cmdGen = None
        
        # set snmp_get to desired method, using function creator method snmp_getter
        self.snmp_get = self.snmp_getter(method=self.method)
        
    def snmp_getter(self, method='pysnmp'):
        '''Returns snmp_get function.  Supply method='cli' to switch to CLI snmpget tool.'''
        
        if method=='pysnmp':
            return self._pysnmp_get(self.hostname, self.port, self.protocol, self.community, 
                                    self.retries, self.timeout)
        else:
            return self._cli_get(self.hostname, self.port, self.protocol, self.community, 
                                    self.retries, self.timeout)

    def _cli_get(self, hostname, port, protocol, community, retries, timeout):
        '''constructs a pysnmp-based snmp_get function with baked-in parameters.
        
        :return: function snmp_get(oid) 
        '''
        def snmp_get(oid):
            snmpproc = Popen(['snmpget',
                          '-v', protocol,
                          '-c', community,
                          '-r', retries,
                          '-t', timeout,
                          '-OvQ',
                          '%s:%s' % (hostname, port),
                          oid],
                         stdout=PIPE)    
            value = snmpproc.communicate()[0].strip()
            return value.strip('"')
        return snmp_get

    def _pysnmp_get(self, hostname, port, protocol, community, retries, timeout):   
        '''returns a pysnmp-based snmp_get function with baked-in parameters.
        
        :return: function snmp_get(oid) 
        '''
        _cmdGen = cmdgen.CommandGenerator()

        def snmp_get(oid):
            (errorIndication, errorStatus, 
                errorIndex, varBinds) = _cmdGen.getCmd(
                                        cmdgen.CommunityData(community),
                                        cmdgen.UdpTransportTarget((hostname, port), 
                                        timeout=timeout, retries=retries), 
                                        oid)
            if errorIndication:
                raise Exception(errorIndication)

            if errorStatus:
                raise Exception('%s at %s\n' % 
                           (errorStatus.prettyPrint(),
                            errorIndex and varBinds[int(errorIndex)-1] or '?'))                       
            return '%s' % varBinds[0][1]
        return snmp_get
        
        

# BEHOLD THE MAGIC SNMP INCANTATIONS
#
# Retrieve a single value:
#
# snmpget -v 2c -c public -OvQ sensaphone1.458.locusdev.net:161 .1.3.6.1.4.1.8338.1.1.4.1.1.2.15
# or
# snmpget -v2c -c public -ObentU sensaphone1.458.locusdev.net 1.3.6.1.4.1.8338.1.1.4.1.1.1.2.15
#
# Walk the whole sensaphone:
#
# snmpwalk -v 2c -m ALL -Os -c public sensaphone1.458.locusdev.net:161 1.3.6.1.4.1.8338.1.1.4



from json import loads,dumps

from sensorcheck import SensorCheck
from sensorhub import SensorHub
from transports.snmp import SNMPobject

DEFAULT_SAMPLERATE = 500.0      # 500 seconds.

class Experiment(object):

    """basic Experiment object.

        keyword argument set attributes:

        name            -- (string)   human-friendly string describing experiment (default: '')
        start           -- (datetime) time at which to kick off experiment (default: None)
        end             -- (datetime) time at which to end experiment (default: None)
        savegame        -- (bool) whether to allow state save / reload (default: True)
        samplerate      -- (float) interval in seconds (default: %f)
        
        attributes set during operation:
        
        savegame_file   -- (string) path to state file 
        
    """ % DEFAULT_SAMPLERATE

    def __init__(self, *args, **kwargs):
    
        self.name = kwargs.get('name', '')
        self.start = kwargs.get('start', datetime.now())
        self.end = kwargs.get('end', None)
        self.savegame = kwargs.get('savegame', True)
        self.samplerate = float(kwargs.get('samplerate', DEFAULT_SAMPLERATE))
        
        self.savegame_file = ''

    @classmethod
    def from_savegame(cls, statefile, **kwargs):
        state = loads(statefile)
        
        meta = state['meta']
        print meta
    
        return cls(savegame=True, **meta)

    def run():
        "Like main() in a command line script -- makes experiment go."
        if self.start == None:
            self.start = datetime.now()
        print "Running!"

    def save():
        "Save state, enabling us to shut down experiment and start later where we left off."
        raise NotImplementedError
        
    def load():
        "Load state, picking up experiment where we left off."
        raise NotImplementedError


class SensorBatteryExperiment(Experiment):

    """SensorBatteryExperiment object (inherits from Experiment)."""

    def __init__(self, idx, hostname, *args, **kwargs):
        self.idx = idx
        self.hostname = hostname

        self.snmp_get = SNMPobject(hostname).snmp_getter()

        self.state_oids = { 'battery_status': get_sensor_oid('intPower'),
                            'measurement': get_sensor_oid('measurement'),
                            'units': get_sensor_oid('units'),
                            'measurement_type': get_sensor_oid('measurement_type'),
                          }
        self.timeseries_data = { }

    def get_status(self):
        for codename in list(timeseries_data.keys()):
            self.timeseries_data[codename] = self.snmp_get(self.state_oids[codename])
        if self.savegame:
            self.save()

    def poll(self):
        "use SNMP to check availability and alarm status of sensor."
        self.statuses.append(  self.get_status() ) 

    def run(self):
        while datetime.now() < self.end:
            poll()
            time.sleep(self.interval)

    def to_dataframe(self):
        from pandas import DataFrame
        return DataFrame(self.timeseries_data)



__doc__='''LabSensor class for abstracting access to lab sensor data via SNMP and nagios.'''
__author__='nthmost'

from datetime import datetime
import json

# TODO: make equipment swappable (when more equipment presents itself)
from equipment.wsg30 import get_sensor_oid
from transports.snmp import SNMPobject
from utils import CustomJsonEncoder, CustomJsonDecoder

class SensorCheck(object):
    '''Represents a single moment of "Sensor Check", sampling data via either SNMP or nagios.

	This class was designed to be used mostly through SensorHub.
	
	Required arguments:
	
	hostname		-- (string) fqdn or LAN-accessible hostname (no prefix like http://)
	idx				-- (int) index of sensor on hostname (0 to MAX_SENSORS_PER_HUB)
	
	Object attributes retrieved by SNMP:

    serial          -- (string) unique serial number of recognized sensor.
    name            -- (string) the human-created name written into the sensor hub.
    measurement     -- (float) current value measured by sensor (generally the most important datum).
    units           -- (string) units for measurement data, e.g. "C" or "F" for temperature.
    alarmhigh       -- (float) value set on sensor hub for high value at which to report alarm.
    alarmlow        -- (float) value set on sensor hub for low value at which to report alarm.
    lastalarm	    -- (datetime) date+time at which sensor last had an alarm condition.
    measurement_type-- (string) Sensaphone set string describing type of measurement (e.g. 'Temp 2.8k C')
    alarm_status    -- (string) Reports as 'OK', 'High', 'Low', 'Low Battery' or 'Trouble' (no response)
    
	Available keyword arguments in addition to all of the above:

    timeout         -- (int or float) time in seconds to wait for SNMP response.
    checktime		-- (datetime) date+time at which measurement data was acquired.
    retries			-- (int) number of times to retry connection (default: 0)
    auto            -- (bool) whether to perform snmp checks immediately (use .check() when ready) 
    ''' 

    @classmethod
    def from_snmp(cls, hostname, idx, auto=True, **kwargs):
        '''constructs a SensorCheck object from SNMP data.
        
        This classmethod is expected to be used mainly from within the SensorHub class,
        but could potentially be used to do an individual sensor check when the idx 
        ("index") of the sensor on the hub is already known.
        
        To get a list of available sensors by index, see SensorHub.sensors_by_idx
        
        :param hostname: (required) hostname of hub to which sensor is attached.
        :param idx: (required) the integer "index" of the sensor on the hub. 
        :param auto: (optional) whether to perform a check immediately (default: True)
        '''
        return cls(hostname=hostname, idx=idx, auto=auto, **kwargs)
        
    @classmethod
    def from_record(cls, record, **kwargs):
        '''constructs a SensorCheck object from a CSV record (exported from equipment).
        
        :param record: (required) csv line containing SensorCheck data.
        '''
        
        CSVFORMAT = 'idx,measurement,checktime'
        
        raise NotImplementedError
        # TODO: make this actually work.
        #return cls(cls, auto=False, **kwargs)
        
    @classmethod
    def from_json(cls, jdoc):
        # use CustomJsonDecoder to convert checktime from isoformat back to python datetime
        jdict = json.loads(jdoc, cls=CustomJsonDecoder)
        return cls(**jdict)
    
    def __init__(self, *args, **kwargs):
        '''SensorCheck base constructor.
        
        Initializes all instance variables both supplied and implied (defaults: None).

        See class documentation for all available arguments.
        '''

        self.name = kwargs.get('name', None)
        self.measurement = kwargs.get('measurement', None)
        self.units = kwargs.get('units', None)
        self.alarmhigh = kwargs.get('alarmhigh', None)
        self.alarmlow = kwargs.get('alarmlow', None)
        self.lastalarm = kwargs.get('lastalarm', None)
        self.alarm_status = kwargs.get('alarm_status', None)
        self.measurement_type = kwargs.get('measurement_type', None)
        self.serial = kwargs.get('serial', None)

        ## SNMP ##        
        self.hostname = kwargs.get('hostname', None)
        self.idx = kwargs.get('idx', None)
        self.timeout = kwargs.get('timeout', 10)
        self.snmp_method = kwargs.get('method', 'pysnmp')
        self.retries = kwargs.get('retries', 0)
        
        auto = kwargs.get('auto', True)
        
        if self.hostname:
    	    self.snmp_get = SNMPobject(hostname=self.hostname, timeout=self.timeout, 
    				method=self.snmp_method, retries=self.retries).snmp_getter()	    
	    
            if auto and self.idx:
    	        self.check()
    	        return

    def check(self):
        '''Uses object's snmp_get method to populate sensor variables, starting by
           acquiring individual datapoint names via get_sensor_oid function.
        '''
        idx = self.idx
        
        self.name = self.snmp_get(get_sensor_oid(idx, 'name'))
        self.units = self.snmp_get(get_sensor_oid(idx, 'units'))
        
        self.measurement = self.snmp_get(get_sensor_oid(idx, 'measurement'))
        try:
            self.measurement = float(self.measurement)
        except ValueError:
            # we have a string, probably an 'N/A'
            pass
            
        self.alarmlow = float(self.snmp_get(get_sensor_oid(idx, 'alarmlow')))
        self.alarmhigh = float(self.snmp_get(get_sensor_oid(idx, 'alarmhigh')))
        self.alarm_status = self.snmp_get(get_sensor_oid(idx, 'alarm_status'))
        self.lastalarm = self.snmp_get(get_sensor_oid(idx, 'lastalarm'))
        self.measurement_type = self.snmp_get(get_sensor_oid(idx, 'measurement_type'))
        self.serial = self.snmp_get(get_sensor_oid(idx, 'serial'))
        self.checktime = datetime.utcnow()
        
        # turned off until I can confirm or discard "battery_level" as a thing.
        #self.battery_level = self.snmp_get(get_sensor_oid(idx, 'battery_level'))

    def to_dict(self):
        return { 'idx': self.idx,
                 'name':  self.name,
                 'measurement':  self.measurement,
                 'units': self.units,
                 'alarmhigh': self.alarmhigh,
                 'alarmlow':  self.alarmlow,
                 'checktime': self.checktime,
                 'alarm_status': self.alarm_status,
                 'lastalarm': self.lastalarm,
                 'hostname':  self.hostname,
                 'measurement_type': self.measurement_type,
                 'serial': self.serial,
                }

    def to_json(self):
        return json.dumps(self.to_dict(), cls=CustomJsonEncoder)

    def __str__(self):
        out = "SensorCheck for Sensor %i at %s\n" % (self.idx, self.hostname)
        out += '%r' % self.to_dict()
        out += '\n'
        return out
        